package com.nd.share.demo;

import com.alibaba.fastjson.JSONObject;
import com.nd.openapi.client.NdHttpClientBuilder;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpHost;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.util.EntityUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;

/**
 * @author wangw
 * @date 2020/4/10
 */
@RestController
@RequestMapping("/open")
public class OpenController {

    private static final String API_GATEWAY_HOST = "https://kaifangpingtai.gw.101.com";

    private static final String UNIFIED_DOMAIN = "http://kaifangpingtai-jichushuju.site.101.com";

    private static final String SDP_APP_ID = "26bab49c-c1b6-45c7-a1e4-2223ca0ee7b1";

    private static final String ACCESS_KEY_ID = "openplatformf46d0f58-8fc8-4881-ac8a-6c2529f5f36e";

    private static final String SECRET_ACCESS_ID = "b0f814fd-21c8-4cf4-9108-f06eecbb7fdd";

    /**
     * 获取授权码页面url
     *
     * @return
     */
    @RequestMapping(value = "/oauthUrl", method = RequestMethod.GET)
    public String oauthUrl() {
        return UNIFIED_DOMAIN + "/openplatform/connect?sdp-app-id=" + SDP_APP_ID + "&response_type=code&access_key_id=" + ACCESS_KEY_ID + "&redirect_uri=http://www.baidu.com&scope=scope_base&state=1";
    }

    /**
     * 根据授权码获取访问令牌
     *
     * @param code
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/authByCode", method = RequestMethod.GET)
    public Object auth(@RequestParam("code") String code) throws IOException {
        if (StringUtils.isBlank(code)) {
            return "code is null";
        }
        HttpClient httpClient = NdHttpClientBuilder.getHttpClient(ACCESS_KEY_ID, SECRET_ACCESS_ID);//此类由sdk提供
        JSONObject obj = new JSONObject();
        obj.put("access_key_id", ACCESS_KEY_ID);
        obj.put("secret_access_key", SECRET_ACCESS_ID);
        obj.put("code", code);
        obj.put("grant_type", "authorization_code");
        HttpPost request = new HttpPost(API_GATEWAY_HOST + "/oauth/access_token");
        request.setHeader("Content-Type", "application/json");
        StringEntity entity = new StringEntity(obj.toString());
        request.setHeader("sdp-app-id", SDP_APP_ID);
        request.setEntity(entity);
        HttpHost httpHost = HttpHost.create(API_GATEWAY_HOST);
        HttpResponse httpResponse = httpClient.execute(httpHost, request);
        if (httpResponse.getStatusLine().getStatusCode() == 200) {
            String result = EntityUtils.toString(httpResponse.getEntity());
            return JSONObject.parseObject(result);
        }
        return EntityUtils.toString(httpResponse.getEntity());
    }

    /**
     * 根据ticket获取访问令牌
     *
     * @param ticket
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/authByTicket", method = RequestMethod.GET)
    public Object authTicket(@RequestParam("ticket") String ticket) throws IOException {
        if (StringUtils.isBlank(ticket)) {
            return "ticket is null";
        }
        HttpClient httpClient = NdHttpClientBuilder.getHttpClient(ACCESS_KEY_ID, SECRET_ACCESS_ID);//此类由sdk提供
        JSONObject obj = new JSONObject();
        obj.put("access_key_id", ACCESS_KEY_ID);
        obj.put("secret_access_key", SECRET_ACCESS_ID);
        obj.put("ticket", ticket);
        obj.put("grant_type", "ticket");
        HttpPost request = new HttpPost(API_GATEWAY_HOST + "/oauth/access_token");
        request.setHeader("Content-Type", "application/json");
        StringEntity entity = new StringEntity(obj.toString());
        request.setHeader("sdp-app-id", SDP_APP_ID);
        request.setEntity(entity);
        HttpHost httpHost = HttpHost.create(API_GATEWAY_HOST);
        HttpResponse httpResponse = httpClient.execute(httpHost, request);
        if (httpResponse.getStatusLine().getStatusCode() == 200) {
            String result = EntityUtils.toString(httpResponse.getEntity());
            return JSONObject.parseObject(result);
        }
        return EntityUtils.toString(httpResponse.getEntity());
    }

    public Object refresh(String refreshToken) throws IOException {
        HttpClient httpClient = NdHttpClientBuilder.getHttpClient(ACCESS_KEY_ID, SECRET_ACCESS_ID);//此类由sdk提供
        JSONObject obj = new JSONObject();
        obj.put("access_key_id", ACCESS_KEY_ID);
        obj.put("refresh_token", refreshToken);
        obj.put("grant_type", "refresh_token");
        HttpPost request = new HttpPost(API_GATEWAY_HOST + "/oauth/refresh_token");
        request.setHeader("Content-Type", "application/json");
        StringEntity entity = new StringEntity(obj.toString());
        request.setHeader("sdp-app-id", SDP_APP_ID);
        request.setEntity(entity);
        HttpHost httpHost = HttpHost.create(API_GATEWAY_HOST);
        HttpResponse httpResponse = httpClient.execute(httpHost, request);
        if (httpResponse.getStatusLine().getStatusCode() == 200) {
            String result = EntityUtils.toString(httpResponse.getEntity());
            return JSONObject.parseObject(result);
        }
        return EntityUtils.toString(httpResponse.getEntity());
    }

    /**
     * 获取用户公开信息
     *
     * @param openid
     * @param accessToken
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/getUserInfo", method = RequestMethod.GET)
    public Object getUserInfo(@RequestParam("openid") String openid, @RequestParam("accessToken") String accessToken) throws IOException {
        HttpClient httpClient = NdHttpClientBuilder.getHttpClient(ACCESS_KEY_ID, SECRET_ACCESS_ID);//此类由sdk提供
        JSONObject obj = new JSONObject();
        obj.put("open_id", openid);
        obj.put("access_token", accessToken);
        HttpPost request = new HttpPost(API_GATEWAY_HOST + "/base/get_user_info");
        request.setHeader("Content-Type", "application/json");
        StringEntity entity = new StringEntity(obj.toString());
        request.setHeader("sdp-app-id", SDP_APP_ID);
        request.setEntity(entity);
        HttpHost httpHost = HttpHost.create(API_GATEWAY_HOST);
        HttpResponse httpResponse = httpClient.execute(httpHost, request);
        if (httpResponse.getStatusLine().getStatusCode() == 200) {
            String result = EntityUtils.toString(httpResponse.getEntity());
            return JSONObject.parseObject(result);
        }
        return EntityUtils.toString(httpResponse.getEntity());
    }

    /**
     * 获取用户信息,包含user_id
     *
     * @param openid
     * @param accessToken
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/getUser", method = RequestMethod.GET)
    public Object getUser(@RequestParam("openid") String openid, @RequestParam("accessToken") String accessToken) throws IOException {
        HttpClient httpClient = NdHttpClientBuilder.getHttpClient(ACCESS_KEY_ID, SECRET_ACCESS_ID);//此类由sdk提供
        HttpGet request = new HttpGet(API_GATEWAY_HOST + "/base/user");
        request.setHeader("Content-Type", "application/json");
        request.setHeader("sdp-app-id", SDP_APP_ID);
        request.setHeader("idp-open-id", openid);
        request.setHeader("idp-access-token", accessToken);
        HttpHost httpHost = HttpHost.create(API_GATEWAY_HOST);
        HttpResponse httpResponse = httpClient.execute(httpHost, request);
        if (httpResponse.getStatusLine().getStatusCode() == 200) {
            String result = EntityUtils.toString(httpResponse.getEntity());
            return JSONObject.parseObject(result);
        }
        return EntityUtils.toString(httpResponse.getEntity());
    }

    /**
     * 学科查询
     *
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/subjects", method = RequestMethod.GET)
    public Object subjects() throws IOException {
        HttpClient httpClient = NdHttpClientBuilder.getHttpClient(ACCESS_KEY_ID, SECRET_ACCESS_ID);//此类由sdk提供
        HttpGet request = new HttpGet(API_GATEWAY_HOST + "/base/subjects");
        request.setHeader("Content-Type", "application/json");
        request.setHeader("sdp-app-id", SDP_APP_ID);
        HttpHost httpHost = HttpHost.create(API_GATEWAY_HOST);
        HttpResponse httpResponse = httpClient.execute(httpHost, request);
        if (httpResponse.getStatusLine().getStatusCode() == 200) {
            String result = EntityUtils.toString(httpResponse.getEntity());
            return JSONObject.parseArray(result);
        }
        return EntityUtils.toString(httpResponse.getEntity());
    }

    /**
     * 学段查询
     *
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/sections", method = RequestMethod.GET)
    public Object sections() throws IOException {
        HttpClient httpClient = NdHttpClientBuilder.getHttpClient(ACCESS_KEY_ID, SECRET_ACCESS_ID);//此类由sdk提供
        HttpGet request = new HttpGet(API_GATEWAY_HOST + "/base/sections");
        request.setHeader("Content-Type", "application/json");
        request.setHeader("sdp-app-id", SDP_APP_ID);
        HttpHost httpHost = HttpHost.create(API_GATEWAY_HOST);
        HttpResponse httpResponse = httpClient.execute(httpHost, request);
        if (httpResponse.getStatusLine().getStatusCode() == 200) {
            String result = EntityUtils.toString(httpResponse.getEntity());
            return JSONObject.parseArray(result);
        }
        return EntityUtils.toString(httpResponse.getEntity());
    }

    /**
     * 用户增量数据
     *
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/modifyUsersActionSearch", method = RequestMethod.GET)
    public Object modifyUsersActionSearch(
            @RequestParam("offset") Long offset,
            @RequestParam("limit") Long limit,
            @RequestParam("orgId") Long orgId,
            @RequestParam("operate") String operate,
            @RequestParam("statPeriodStartDate") Long statPeriodStartDate,
            @RequestParam("statPeriodEndDate") Long statPeriodEndDate) throws IOException {
        HttpClient httpClient = NdHttpClientBuilder.getHttpClient(ACCESS_KEY_ID, SECRET_ACCESS_ID);//此类由sdk提供
        HttpPost request = new HttpPost(API_GATEWAY_HOST + "/base/modify_users/action/search");
        request.setHeader("Content-Type", "application/json");
        request.setHeader("sdp-app-id", SDP_APP_ID);
        JSONObject obj = new JSONObject();
        obj.put("offset", offset);//分页索引
        obj.put("limit", limit);//每页数量
        obj.put("org_id", orgId);//组织id（学校id）
        obj.put("operate", operate);//操作类型：CREATE、UPDATE、DELETE 不传为全部操作
        obj.put("stat_period_start_date", statPeriodStartDate);//操作开始时间：格式yyyyMMdd
        obj.put("stat_period_end_date", statPeriodEndDate);//操作结束时间：格式yyyyMMdd
        StringEntity entity = new StringEntity(obj.toString());
        request.setEntity(entity);
        HttpHost httpHost = HttpHost.create(API_GATEWAY_HOST);
        HttpResponse httpResponse = httpClient.execute(httpHost, request);
        if (httpResponse.getStatusLine().getStatusCode() == 200) {
            String result = EntityUtils.toString(httpResponse.getEntity());
            return JSONObject.parseObject(result);
        }
        return EntityUtils.toString(httpResponse.getEntity());
    }

    /**
     * 查询学校列表
     *
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/schools", method = RequestMethod.GET)
    public Object schools(@RequestParam("offset") Integer offset, @RequestParam("limit") Integer limit, @RequestParam("name") String name) throws IOException {
        HttpClient httpClient = NdHttpClientBuilder.getHttpClient(ACCESS_KEY_ID, SECRET_ACCESS_ID);//此类由sdk提供
        HttpPost request = new HttpPost(String.format(API_GATEWAY_HOST + "/base/schools/actions/search_all?offset=%s&limit=%s", offset, limit));
        request.setHeader("Content-Type", "application/json");
        request.setHeader("sdp-app-id", SDP_APP_ID);
        JSONObject obj = new JSONObject();
        obj.put("name", name);
        StringEntity entity = new StringEntity(obj.toString());
        request.setEntity(entity);
        HttpHost httpHost = HttpHost.create(API_GATEWAY_HOST);
        HttpResponse httpResponse = httpClient.execute(httpHost, request);
        if (httpResponse.getStatusLine().getStatusCode() == 200) {
            String result = EntityUtils.toString(httpResponse.getEntity());
            return JSONObject.parseArray(result);
        }
        return EntityUtils.toString(httpResponse.getEntity());
    }

    /**
     * 查询用户有权限的学校列表
     *
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/schoolsPrivate", method = RequestMethod.GET)
    public Object schoolsPrivate(
            @RequestParam("offset") Integer offset,
            @RequestParam("limit") Integer limit,
            @RequestParam("name") String name,
            String openid,
            String accessToken) throws IOException {
        HttpClient httpClient = NdHttpClientBuilder.getHttpClient(ACCESS_KEY_ID, SECRET_ACCESS_ID);//此类由sdk提供
        HttpPost request = new HttpPost(String.format(API_GATEWAY_HOST + "/base/schools/actions/search_all?offset=%s&limit=%s", offset, limit));
        request.setHeader("Content-Type", "application/json");
        request.setHeader("sdp-app-id", SDP_APP_ID);
        request.setHeader("idp-open-id", openid);
        request.setHeader("idp-access-token", accessToken);
        JSONObject obj = new JSONObject();
        obj.put("name", name);
        StringEntity entity = new StringEntity(obj.toString());
        request.setEntity(entity);
        HttpHost httpHost = HttpHost.create(API_GATEWAY_HOST);
        HttpResponse httpResponse = httpClient.execute(httpHost, request);
        if (httpResponse.getStatusLine().getStatusCode() == 200) {
            String result = EntityUtils.toString(httpResponse.getEntity());
            return JSONObject.parseArray(result);
        }
        return EntityUtils.toString(httpResponse.getEntity());
    }

    /**
     * 查询年级列表
     *
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/grades", method = RequestMethod.GET)
    public static Object grades(long schoolId, String section) throws IOException {
        HttpClient httpClient = NdHttpClientBuilder.getHttpClient(ACCESS_KEY_ID, SECRET_ACCESS_ID);//此类由sdk提供
        HttpGet request = new HttpGet(String.format(API_GATEWAY_HOST + "/base/schools/%s/grades?section=%s", schoolId, section));
        request.setHeader("Content-Type", "application/json");
        request.setHeader("sdp-app-id", SDP_APP_ID);
        HttpHost httpHost = HttpHost.create(API_GATEWAY_HOST);
        HttpResponse httpResponse = httpClient.execute(httpHost, request);
        if (httpResponse.getStatusLine().getStatusCode() == 200) {
            String result = EntityUtils.toString(httpResponse.getEntity());
            return JSONObject.parseArray(result);
        }
        return EntityUtils.toString(httpResponse.getEntity());
    }

    /**
     * 查询单个年级信息
     *
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/grade", method = RequestMethod.GET)
    public static Object grade(long schoolId, long gradeId) throws IOException {
        HttpClient httpClient = NdHttpClientBuilder.getHttpClient(ACCESS_KEY_ID, SECRET_ACCESS_ID);//此类由sdk提供
        HttpGet request = new HttpGet(String.format(API_GATEWAY_HOST + "/base/schools/%s/grades/%s", schoolId, gradeId));
        request.setHeader("Content-Type", "application/json");
        request.setHeader("sdp-app-id", SDP_APP_ID);
        HttpHost httpHost = HttpHost.create(API_GATEWAY_HOST);
        HttpResponse httpResponse = httpClient.execute(httpHost, request);
        if (httpResponse.getStatusLine().getStatusCode() == 200) {
            String result = EntityUtils.toString(httpResponse.getEntity());
            return JSONObject.parseObject(result);
        }
        return EntityUtils.toString(httpResponse.getEntity());
    }

//    public static void main(String[] args) throws IOException {
//        System.out.println(authTicket("1065a8b9-897c-47e2-8c74-4b9c762661f6"));
//        System.out.println(getUserInfo("67d7cdeb-5155-47ed-88a0-620d640f811b","11D6E8D8253152A5E01EB5BA3DE4C2086115537434945506FB73B179CBC1DEDBC52A65905D16AC6C6B8869552EB76C8C0B28FBE677107A66B23A29612235CEA825166A04728862B3CBA533CFD61B691D"));
//        System.out.println(getUser("d7cac7c2-b8a7-4739-aa93-04d6fa756215","11D6E8D8253152A5393853F3EDF31DDA3FC13EA6BE9135063793AA2BDF055902426AA5E1DB6F4DDBC984C7F3CD12A1D49984044D396C0B9A16D2CB1E6B68949D80ECFF37807A718CCA8464AB67B85D90"));
//        System.out.println(getUserInfo("c50b8410-bb55-4b78-a507-d788dec24e72","11D6E8D8253152A5B681DB3FD461A87A622E239F28D48B902B841B0D8E3E2C114A61106A8223BF16F7E0E031BC6E15D2529D69299046CDCBBADDA00B2BA6353F495201578E53D43B5CE072A97F1F9C22"));
//        System.out.println(subjects());
//        System.out.println(sections());
//        System.out.println(schools(0, 20, null));
//        System.out.println(schoolsPrivate(0, 20, null, "d7cac7c2-b8a7-4739-aa93-04d6fa756215", "11D6E8D8253152A5393853F3EDF31DDA3FC13EA6BE9135063793AA2BDF055902426AA5E1DB6F4DDBC984C7F3CD12A1D43453C0EADB9073B525C64D2BCC668B2844AAFBAF1AFBAC80D5BED5DB079AE6D6"));
//    }

}
